'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var sync = require('framesync');
var sync__default = _interopDefault(sync);
var popmotion = require('popmotion');
var heyListen = require('hey-listen');
var styleValueTypes = require('style-value-types');
var styler = require('stylefire');
var styler__default = _interopDefault(styler);
var popcorn = require('@popmotion/popcorn');
var React = require('react');

var htmlTuple = function (args) { return args; };
var svgTuple = function (args) { return args; };
var htmlElements = htmlTuple([
    "a",
    "abbr",
    "address",
    "area",
    "article",
    "aside",
    "audio",
    "b",
    "base",
    "bdi",
    "bdo",
    "big",
    "blockquote",
    "body",
    "br",
    "button",
    "canvas",
    "caption",
    "cite",
    "code",
    "col",
    "colgroup",
    "data",
    "datalist",
    "dd",
    "del",
    "details",
    "dfn",
    "dialog",
    "div",
    "dl",
    "dt",
    "em",
    "embed",
    "fieldset",
    "figcaption",
    "figure",
    "footer",
    "form",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "head",
    "header",
    "hgroup",
    "hr",
    "html",
    "i",
    "iframe",
    "img",
    "input",
    "ins",
    "kbd",
    "keygen",
    "label",
    "legend",
    "li",
    "link",
    "main",
    "map",
    "mark",
    "menu",
    "menuitem",
    "meta",
    "meter",
    "nav",
    "noscript",
    "object",
    "ol",
    "optgroup",
    "option",
    "output",
    "p",
    "param",
    "picture",
    "pre",
    "progress",
    "q",
    "rp",
    "rt",
    "ruby",
    "s",
    "samp",
    "script",
    "section",
    "select",
    "small",
    "source",
    "span",
    "strong",
    "style",
    "sub",
    "summary",
    "sup",
    "table",
    "tbody",
    "td",
    "textarea",
    "tfoot",
    "th",
    "thead",
    "time",
    "title",
    "tr",
    "track",
    "u",
    "ul",
    "var",
    "video",
    "wbr",
    "webview",
]);
var svgElements = svgTuple([
    "animate",
    "circle",
    "clipPath",
    "defs",
    "desc",
    "ellipse",
    "feBlend",
    "feColorMatrix",
    "feComponentTransfer",
    "feComposite",
    "feConvolveMatrix",
    "feDiffuseLighting",
    "feDisplacementMap",
    "feDistantLight",
    "feDropShadow",
    "feFlood",
    "feFuncA",
    "feFuncB",
    "feFuncG",
    "feFuncR",
    "feGaussianBlur",
    "feImage",
    "feMerge",
    "feMergeNode",
    "feMorphology",
    "feOffset",
    "fePointLight",
    "feSpecularLighting",
    "feSpotLight",
    "feTile",
    "feTurbulence",
    "filter",
    "foreignObject",
    "g",
    "image",
    "line",
    "linearGradient",
    "marker",
    "mask",
    "metadata",
    "path",
    "pattern",
    "polygon",
    "polyline",
    "radialGradient",
    "rect",
    "stop",
    "svg",
    "switch",
    "symbol",
    "text",
    "textPath",
    "tspan",
    "use",
    "view",
]);
var elements = htmlElements.concat(svgElements);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

var isFloat = function (value) {
    return !isNaN(parseFloat(value));
};
var MotionValue = (function () {
    function MotionValue(init, _a) {
        var _b = _a === void 0 ? {} : _a, transformer = _b.transformer, parent = _b.parent;
        var _this = this;
        this.timeDelta = 0;
        this.lastUpdated = 0;
        this.cancelSubscriber = new Set();
        this.canTrackVelocity = false;
        this.notifySubscriber = function (subscriber) {
            subscriber(_this.current);
        };
        this.scheduleVelocityCheck = function () { return sync__default.postRender(_this.velocityCheck); };
        this.velocityCheck = function (_a) {
            var timestamp = _a.timestamp;
            if (timestamp !== _this.lastUpdated) {
                _this.prev = _this.current;
            }
        };
        this.setChild = function (child) { return child.set(_this.current); };
        this.parent = parent;
        this.transformer = transformer;
        this.set(init);
        this.canTrackVelocity = isFloat(this.current);
    }
    MotionValue.prototype.addChild = function (config) {
        var child = new MotionValue(this.current, __assign({ parent: this }, config));
        if (!this.children)
            this.children = new Set();
        this.children.add(child);
        return child;
    };
    MotionValue.prototype.removeChild = function (child) {
        this.children.delete(child);
    };
    MotionValue.prototype.addUpdateSubscription = function (subscription) {
        var _this = this;
        if (!this.updateSubscribers)
            this.updateSubscribers = new Set();
        var updateSubscriber = function () { return subscription(_this.current); };
        var scheduleUpdate = function () { return sync__default.update(updateSubscriber, false, true); };
        this.updateSubscribers.add(scheduleUpdate);
        this.cancelSubscriber.add(function () { return sync.cancelSync.update(updateSubscriber); });
        return function () { return _this.updateSubscribers.delete(scheduleUpdate); };
    };
    MotionValue.prototype.addRenderSubscription = function (subscription) {
        var _this = this;
        if (!this.renderSubscribers)
            this.renderSubscribers = new Set();
        var updateSubscriber = function () { return subscription(_this.current); };
        var scheduleUpdate = function () { return sync__default.render(updateSubscriber); };
        this.renderSubscribers.add(scheduleUpdate);
        scheduleUpdate();
        this.cancelSubscriber.add(function () { return sync.cancelSync.render(updateSubscriber); });
        return function () { return _this.renderSubscribers.delete(scheduleUpdate); };
    };
    MotionValue.prototype.set = function (v, render) {
        if (render === void 0) { render = true; }
        this.prev = this.current;
        this.current = this.transformer ? this.transformer(v) : v;
        if (this.updateSubscribers) {
            this.updateSubscribers.forEach(this.notifySubscriber);
        }
        if (this.children) {
            this.children.forEach(this.setChild);
        }
        if (render && this.renderSubscribers) {
            this.renderSubscribers.forEach(this.notifySubscriber);
        }
        var _a = sync.getFrameData(), delta = _a.delta, timestamp = _a.timestamp;
        this.timeDelta = delta;
        this.lastUpdated = timestamp;
        sync__default.postRender(this.scheduleVelocityCheck);
    };
    MotionValue.prototype.get = function () {
        return this.current;
    };
    MotionValue.prototype.getVelocity = function () {
        return this.canTrackVelocity
            ?
                popcorn.velocityPerSecond(parseFloat(this.prev) - parseFloat(this.current), this.timeDelta)
            : 0;
    };
    MotionValue.prototype.control = function (controller, _a, transformer) {
        var _this = this;
        var delay = _a.delay, config = __rest(_a, ["delay"]);
        this.stop();
        var initialisedController = controller(__assign({ from: this.get(), velocity: this.getVelocity() }, config));
        if (transformer) {
            initialisedController = initialisedController.pipe(transformer);
        }
        if (delay) {
            initialisedController = popmotion.chain(popmotion.delay(delay), initialisedController);
        }
        return new Promise(function (complete) {
            _this.controller = initialisedController.start({
                update: function (v) { return _this.set(v); },
                complete: complete,
            });
        });
    };
    MotionValue.prototype.stop = function () {
        if (this.controller)
            this.controller.stop();
    };
    MotionValue.prototype.destroy = function () {
        this.updateSubscribers.clear();
        this.renderSubscribers.clear();
        this.cancelSubscriber.forEach(function (cancel) { return cancel(); });
        this.cancelSubscriber.clear();
        this.parent && this.parent.removeChild(this);
        this.stop();
    };
    return MotionValue;
}());
var motionValue = function (init, opts) { return new MotionValue(init, opts); };

var poseToArray = function (pose) {
    if (!pose) {
        return [];
    }
    if (Array.isArray(pose)) {
        return pose;
    }
    return [pose];
};
var resolvePoses = function (pose) {
    var unresolvedPose = pose instanceof MotionValue ? pose.get() : pose;
    return Array.from(new Set(poseToArray(unresolvedPose)));
};
var asDependencyList = function (list) { return [list.join(",")]; };

var MotionContext = React.createContext({ dragging: false });
var useMotionContext = function (controls, inherit, pose) {
    var _a = React.useContext(MotionContext), parentPose = _a.pose, dragging = _a.dragging;
    var context = React.useMemo(function () { return ({ controls: controls, dragging: dragging }); }, asDependencyList(resolvePoses(pose)));
    context.pose = inherit ? parentPose : pose;
    return context;
};

var MotionValuesMap = (function () {
    function MotionValuesMap() {
        this.hasMounted = false;
        this.values = new Map();
        this.unsubscribers = new Map();
    }
    MotionValuesMap.prototype.has = function (key) {
        return this.values.has(key);
    };
    MotionValuesMap.prototype.set = function (key, value) {
        this.values.set(key, value);
        if (this.hasMounted)
            this.bindValueToStyler(key, value);
    };
    MotionValuesMap.prototype.get = function (key, defaultValue) {
        var value = this.values.get(key);
        if (value === undefined && defaultValue !== undefined) {
            value = new MotionValue(defaultValue);
            this.set(key, value);
        }
        return value;
    };
    MotionValuesMap.prototype.forEach = function (callback) {
        return this.values.forEach(callback);
    };
    MotionValuesMap.prototype.bindValueToStyler = function (key, value) {
        var _this = this;
        var update = function (v) { return _this.styler.set(key, v); };
        var unsubscribe = value.addRenderSubscription(update);
        this.unsubscribers.set(key, unsubscribe);
    };
    MotionValuesMap.prototype.mount = function (element) {
        var _this = this;
        this.hasMounted = true;
        this.styler = styler__default(element);
        this.values.forEach(function (value, key) { return _this.bindValueToStyler(key, value); });
    };
    MotionValuesMap.prototype.unmount = function () {
        var _this = this;
        this.values.forEach(function (_value, key) {
            var unsubscribe = _this.unsubscribers.get(key);
            unsubscribe && unsubscribe();
        });
    };
    return MotionValuesMap;
}());
var useMotionValues = function (ref) {
    var motionValues = React.useMemo(function () { return new MotionValuesMap(); }, []);
    React.useEffect(function () {
        heyListen.invariant(ref.current instanceof Element, "No `ref` found. Ensure components created with `motion.custom` forward refs using `React.forwardRef`");
        motionValues.mount(ref.current);
        return function () { return motionValues.unmount(); };
    });
    return motionValues;
};

var underDampedSpring = function () { return ({
    type: "spring",
    stiffness: 500,
    damping: 25,
    restDelta: 0.5,
    restSpeed: 10,
}); };
var overDampedSpring = function (to) { return ({
    type: "spring",
    stiffness: 700,
    damping: to === 0 ? 100 : 35,
}); };
var linearTween = function () { return ({
    ease: "linear",
    duration: 250,
}); };
var defaultTransitions = {
    x: underDampedSpring,
    y: underDampedSpring,
    z: underDampedSpring,
    rotate: underDampedSpring,
    rotateX: underDampedSpring,
    rotateY: underDampedSpring,
    rotateZ: underDampedSpring,
    scaleX: overDampedSpring,
    scaleY: overDampedSpring,
    scale: overDampedSpring,
    opacity: linearTween,
    backgroundColor: linearTween,
    color: linearTween,
    default: overDampedSpring,
};
var getDefaultTransition = function (valueKey, to) {
    var transitionFactory = defaultTransitions[valueKey] || defaultTransitions.default;
    return __assign({}, transitionFactory(to), { to: to });
};

var just = function (_a) {
    var to = _a.to;
    return popmotion.action(function (_a) {
        var update = _a.update, complete = _a.complete;
        update(to);
        complete();
    });
};
var transitions = { tween: popmotion.tween, spring: popmotion.spring, keyframes: popmotion.keyframes, decay: popmotion.decay, physics: popmotion.physics, just: just };
var linear = popmotion.easing.linear, easeIn = popmotion.easing.easeIn, easeOut = popmotion.easing.easeOut, easeInOut = popmotion.easing.easeInOut, circIn = popmotion.easing.circIn, circOut = popmotion.easing.circOut, circInOut = popmotion.easing.circInOut, backIn = popmotion.easing.backIn, backOut = popmotion.easing.backOut, backInOut = popmotion.easing.backInOut, anticipate = popmotion.easing.anticipate;
var easingLookup = {
    linear: linear,
    easeIn: easeIn,
    easeOut: easeOut,
    easeInOut: easeInOut,
    circIn: circIn,
    circOut: circOut,
    circInOut: circInOut,
    backIn: backIn,
    backOut: backOut,
    backInOut: backInOut,
    anticipate: anticipate,
};
var transitionOptionParser = {
    tween: function (opts) {
        var ease = opts.ease;
        if (Array.isArray(ease)) {
            heyListen.invariant(ease.length === 4, "Cubic bezier arrays must contain four numerical values.");
            var x1 = ease[0], y1 = ease[1], x2 = ease[2], y2 = ease[3];
            opts.ease = popmotion.easing.cubicBezier(x1, y1, x2, y2);
        }
        else if (typeof ease === "string") {
            heyListen.invariant(easingLookup[ease] !== undefined, "Invalid easing type '" + ease + "'");
            opts.ease = easingLookup[ease];
        }
        return opts;
    },
    keyframes: function (_a) {
        var from = _a.from, to = _a.to, opts = __rest(_a, ["from", "to"]);
        return opts;
    },
};
var getPoseTransitions = function (valueKey, to, transitionProp) {
    if (transitionProp !== undefined) {
        var transition = {};
        if (transitionProp === false || transitionProp[valueKey] === false) {
            transition = { type: "just" };
        }
        else {
            transition = transitionProp[valueKey] || transitionProp.default || transitionProp;
        }
        return __assign({}, transition, { to: to });
    }
    return getDefaultTransition(valueKey, to);
};
var preprocessOptions = function (type, opts) {
    return transitionOptionParser[type] ? transitionOptionParser[type](opts) : opts;
};
var getTransition = function (valueKey, to, transition) {
    var _a = getPoseTransitions(valueKey, to, transition), _b = _a.type, type = _b === void 0 ? "tween" : _b, transitionDefinition = __rest(_a, ["type"]);
    var actionFactory = transitions[type];
    var opts = preprocessOptions(type, transitionDefinition);
    return [actionFactory, opts];
};

var isAnimatable = function (value) { return typeof value === "number" || styleValueTypes.complex.test(value); };
var isPoseResolver = function (p) { return typeof p === "function"; };
var isPoseKeyList = function (v) { return Array.isArray(v); };
var setValues = function (target, values, isActive) {
    if (isActive === void 0) { isActive = new Set(); }
    return Object.keys(target).forEach(function (key) {
        if (isActive.has(key))
            return;
        isActive.add(key);
        if (values.has(key)) {
            var value = values.get(key);
            value && value.set(target[key]);
        }
        else {
            values.set(key, motionValue(target[key]));
        }
    });
};
var resolveAnimationDefinition = function (definition, props) {
    if (props === void 0) { props = {}; }
    if (!definition)
        return [];
    if (isPoseResolver(definition)) {
        definition = definition(props);
    }
    return Array.isArray(definition) ? definition : [definition];
};
var AnimationControls = (function () {
    function AnimationControls(values) {
        this.poses = {};
        this.isAnimating = new Set();
        this.values = values;
    }
    AnimationControls.prototype.setProps = function (props) {
        this.props = props;
    };
    AnimationControls.prototype.setPoses = function (poses) {
        this.poses = poses;
    };
    AnimationControls.prototype.start = function (definition, transition) {
        this.resetIsAnimating();
        if (isPoseKeyList(definition)) {
            return this.animateMultiplePoses(definition);
        }
        else if (typeof definition === "string") {
            return this.animatePose(definition);
        }
        else {
            return this.animate([definition, transition]);
        }
    };
    AnimationControls.prototype.set = function (poseKeyList) {
        var _this = this;
        var isSetting = new Set();
        var reversedList = poseKeyList.slice().reverse();
        reversedList.forEach(function (poseKey) {
            var _a = resolveAnimationDefinition(_this.poses[poseKey], _this.props), target = _a[0], transition = _a[1];
            if (transition) {
                var applyOnEnd = transition.applyOnEnd;
                applyOnEnd && setValues(applyOnEnd, _this.values, isSetting);
            }
            target && setValues(target, _this.values, isSetting);
        });
    };
    AnimationControls.prototype.animate = function (animationDefinition, delay) {
        var _this = this;
        if (delay === void 0) { delay = 0; }
        var _a = resolveAnimationDefinition(animationDefinition), target = _a[0], _b = _a[1], transition = _b === void 0 ? {} : _b;
        if (!target)
            return Promise.resolve();
        var animations = Object.keys(target).reduce(function (acc, key) {
            if (_this.isAnimating.has(key))
                return acc;
            if (!_this.values.has(key)) {
                _this.values.set(key, motionValue(0));
            }
            var value = _this.values.get(key);
            if (!value)
                return acc;
            var valueTarget = target[key];
            if (isAnimatable(valueTarget)) {
                var _a = getTransition(key, valueTarget, __assign({ delay: delay }, transition)), action = _a[0], options = _a[1];
                acc.push(value.control(action, options));
            }
            else {
                value.set(valueTarget);
            }
            _this.isAnimating.add(key);
            return acc;
        }, []);
        return Promise.all(animations).then(function () {
            if (!transition || !transition.applyOnEnd)
                return;
            var applyOnEnd = transition.applyOnEnd;
            setValues(applyOnEnd, _this.values);
        });
    };
    AnimationControls.prototype.animateMultiplePoses = function (poseKeyList) {
        var _this = this;
        var animations = poseKeyList.slice().reverse().map(function (poseKey) { return _this.animatePose(poseKey); });
        return Promise.all(animations);
    };
    AnimationControls.prototype.animatePose = function (poseKey, delay) {
        var _this = this;
        if (delay === void 0) { delay = 0; }
        var beforeChildren = false;
        var afterChildren = false;
        var delayChildren = 0;
        var staggerChildren = 0;
        var staggerDirection = 1;
        var pose = this.poses[poseKey];
        var getAnimations = pose ? function () { return _this.animate(pose, delay); } : function () { return Promise.resolve(); };
        var getChildrenAnimations = this.children
            ? function () { return _this.animateChildren(poseKey, delayChildren, staggerChildren, staggerDirection); }
            : function () { return Promise.resolve(); };
        if (pose && this.children) {
            var _a = resolveAnimationDefinition(pose), transition = _a[1];
            if (transition) {
                beforeChildren = transition.beforeChildren || beforeChildren;
                afterChildren = transition.afterChildren || afterChildren;
                delayChildren = transition.delayChildren || delayChildren;
            }
        }
        if (beforeChildren || afterChildren) {
            var _b = beforeChildren
                ? [getAnimations, getChildrenAnimations]
                : [getChildrenAnimations, getAnimations], first = _b[0], last = _b[1];
            return first().then(last);
        }
        else {
            return Promise.all([getAnimations(), getChildrenAnimations()]);
        }
    };
    AnimationControls.prototype.animateChildren = function (poseKey, delayChildren, staggerChildren, staggerDirection) {
        if (delayChildren === void 0) { delayChildren = 0; }
        if (staggerChildren === void 0) { staggerChildren = 0; }
        if (staggerDirection === void 0) { staggerDirection = 1; }
        var animations = [];
        var maxStaggerDuration = (this.children.size - 1) * staggerChildren;
        var generateStaggerDuration = staggerDirection === 1
            ? function (i) { return i * staggerChildren; }
            : function (i) { return maxStaggerDuration - i * staggerChildren; };
        Array.from(this.children).forEach(function (childControls, i) {
            var animation = childControls.animatePose(poseKey, delayChildren + generateStaggerDuration(i));
            animations.push(animation);
        });
        return Promise.all(animations);
    };
    AnimationControls.prototype.resetIsAnimating = function () {
        this.isAnimating.clear();
        if (this.children) {
            this.children.forEach(function (child) { return child.resetIsAnimating(); });
        }
    };
    AnimationControls.prototype.stop = function () {
        this.values.forEach(function (value) { return value.stop(); });
    };
    AnimationControls.prototype.addChild = function (controls) {
        if (!this.children) {
            this.children = new Set();
        }
        this.children.add(controls);
    };
    AnimationControls.prototype.removeChild = function (controls) {
        this.children.delete(controls);
    };
    AnimationControls.prototype.resetChildren = function () {
        if (this.children)
            this.children.clear();
    };
    return AnimationControls;
}());
var useAnimationControls = function (values, inherit, props) {
    var parentControls = React.useContext(MotionContext).controls;
    var controls = React.useMemo(function () { return new AnimationControls(values); }, []);
    controls.resetChildren();
    if (inherit && parentControls) {
        parentControls.addChild(controls);
    }
    React.useEffect(function () { return function () { return parentControls && parentControls.removeChild(controls); }; }, []);
    controls.setProps(props);
    return controls;
};

var useExternalRef = function (external) {
    var ref = !external || typeof external === "function" ? React.useRef(null) : external;
    React.useEffect(function () {
        if (external && typeof external === "function") {
            external(ref.current);
            return function () { return external(null); };
        }
    });
    return ref;
};

var createValueResolver = function (resolver) { return function (values) {
    var resolvedValues = {};
    values.forEach(function (value, key) { return (resolvedValues[key] = resolver(value)); });
    return resolvedValues;
}; };
var resolveCurrent = createValueResolver(function (value) { return value.get(); });

var isMotionValue = function (value) { return value instanceof MotionValue; };
var useStyleAttr = function (values, styleProp) {
    if (styleProp === void 0) { styleProp = {}; }
    var style = {};
    Object.keys(styleProp).forEach(function (key) {
        var thisStyle = styleProp[key];
        if (isMotionValue(thisStyle)) {
            if (values.get(key) !== thisStyle) {
                values.set(key, thisStyle);
            }
        }
        else {
            style[key] = thisStyle;
        }
    });
    return React.useMemo(function () { return (__assign({}, style, styler.buildStyleProperty(resolveCurrent(values)))); }, [style, values]);
};

var AnimationManager = (function () {
    function AnimationManager() {
        this.hasMounted = false;
        this.pendingAnimations = [];
        this.subscribers = new Set();
        this.poses = {};
    }
    AnimationManager.prototype.setPoses = function (poses) {
        this.poses = poses;
        this.subscribers.forEach(function (subscriber) { return subscriber.setPoses(poses); });
    };
    AnimationManager.prototype.subscribe = function (subscriber) {
        var _this = this;
        this.subscribers.add(subscriber);
        if (this.poses)
            subscriber.setPoses(this.poses);
        return function () { return _this.subscribers.delete(subscriber); };
    };
    AnimationManager.prototype.start = function (definition, transition) {
        if (this.hasMounted) {
            var animations_1 = [];
            this.subscribers.forEach(function (controls) {
                var animation = controls.start(definition, transition);
                animations_1.push(animation);
            });
            return Promise.all(animations_1);
        }
        else {
            this.pendingAnimations.push([definition, transition]);
            return Promise.resolve();
        }
    };
    AnimationManager.prototype.stop = function () {
        this.subscribers.forEach(function (subscriber) { return subscriber.stop(); });
    };
    AnimationManager.prototype.mount = function () {
        var _this = this;
        this.hasMounted = true;
        this.pendingAnimations.forEach(function (_a) {
            var definition = _a[0], transition = _a[1];
            return _this.start(definition, transition);
        });
    };
    AnimationManager.prototype.unmount = function () {
        this.hasMounted = false;
        this.stop();
    };
    return AnimationManager;
}());

var isAnimationManager = function (animation) { return animation instanceof AnimationManager; };
var useAnimationSubscription = function (animation, controls) {
    var unsubscribe = React.useMemo(function () { return isAnimationManager(animation) && animation.subscribe(controls); }, [animation]);
    React.useEffect(function () { return function () { return unsubscribe && unsubscribe(); }; }, [unsubscribe]);
};

var isPoses = function (animation) {
    return !(animation instanceof AnimationManager) && animation !== undefined;
};
var usePoses = function (animation, inherit, controls, onPoseComplete, pose, initialPose) {
    var parentPose = React.useContext(MotionContext).pose;
    var poseToResolve = inherit ? parentPose : pose;
    var poseList = resolvePoses(poseToResolve);
    var hasMounted = React.useRef(false);
    var isPoseControlled = isPoses(animation);
    if (isPoseControlled) {
        controls.setPoses(animation);
    }
    React.useMemo(function () {
        if (!isPoseControlled)
            return;
        var initial = initialPose ? resolvePoses(initialPose) : poseList;
        controls.set(initial);
    }, [isPoseControlled]);
    React.useEffect(function () {
        if (inherit || !isPoseControlled)
            return;
        if (hasMounted.current || initialPose) {
            controls.start(poseList).then(function () {
                onPoseComplete && onPoseComplete();
            });
        }
        hasMounted.current = true;
    }, asDependencyList(poseList));
};

function isEventTarget(target) {
    return typeof target.addEventListener === "function" && typeof target.removeEventListener === "function";
}
var eventListener = function (target, name, handler, options) {
    var startListening = function () {
        if (!target) {
            return;
        }
        target.addEventListener(name, handler, options);
    };
    var stopListening = function () {
        if (!target) {
            return;
        }
        target.removeEventListener(name, handler, options);
    };
    return [startListening, stopListening];
};
var useEvent = function (type, target, handler, options) {
    var result = undefined;
    if ((!target || isEventTarget(target)) && handler) {
        result = eventListener(target, type, handler, options);
    }
    React.useEffect(function () {
        var ref = target;
        if (!handler || !target || isEventTarget(target) || !ref.current) {
            return;
        }
        var _a = eventListener(ref.current, type, handler, options), start = _a[0], stop = _a[1];
        start();
        return stop;
    }, [type, target, handler, options]);
    return result;
};

var pointForTarget = function (event, target) {
    if (!target) {
        return { x: event.pageX, y: event.pageY };
    }
    if (window.webkitConvertPointFromPageToNode) {
        var webkitPoint = new WebKitPoint(event.pageX, event.pageY);
        webkitPoint = window.webkitConvertPointFromPageToNode(target, webkitPoint);
        return { x: webkitPoint.x, y: webkitPoint.y };
    }
    var rect = target.getBoundingClientRect();
    var scaleX = 1;
    if (target.style.width && target.style.width !== "") {
        scaleX = parseFloat(target.style.width) / rect.width;
    }
    var scaleY = 1;
    if (target.style.height && target.style.height !== "") {
        scaleY = parseFloat(target.style.height) / rect.height;
    }
    var scale = {
        x: scaleX,
        y: scaleY,
    };
    var point = {
        x: scale.x * (event.pageX - rect.left - target.clientLeft + target.scrollLeft),
        y: scale.y * (event.pageY - rect.top - target.clientTop + target.scrollTop),
    };
    return point;
};
var extractEventInfo = function (event) {
    var target = event.target instanceof HTMLElement ? event.target : null;
    var point = pointForTarget(event, target);
    var devicePoint = pointForTarget(event, document.body);
    return { point: point, devicePoint: devicePoint };
};
var wrapHandler = function (handler) {
    if (!handler) {
        return undefined;
    }
    var listener = function (event, info) {
        if (!info) {
            info = extractEventInfo(event);
        }
        handler(event, info);
    };
    return listener;
};

var mergeUseEventResults = function () {
    var values = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        values[_i] = arguments[_i];
    }
    if (values.every(function (v) { return v === undefined; })) {
        return;
    }
    var start = function () {
        for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
            var value = values_1[_i];
            if (value) {
                value[0]();
            }
        }
    };
    var stop = function () {
        for (var _i = 0, values_2 = values; _i < values_2.length; _i++) {
            var value = values_2[_i];
            if (value) {
                value[1]();
            }
        }
    };
    return [start, stop];
};
var useMouseEvents = function (_a, ref, options) {
    var onMouseDown = _a.onMouseDown, onMouseMove = _a.onMouseMove, onMouseUp = _a.onMouseUp, onMouseOver = _a.onMouseOver, onMouseOut = _a.onMouseOut, onMouseEnter = _a.onMouseEnter, onMouseLeave = _a.onMouseLeave;
    var down = useEvent("mousedown", ref, wrapHandler(onMouseDown), options);
    var move = useEvent("mousemove", ref, wrapHandler(onMouseMove), options);
    var up = useEvent("mouseup", ref, wrapHandler(onMouseUp), options);
    var over = useEvent("mouseover", ref, wrapHandler(onMouseOver), options);
    var out = useEvent("mouseout", ref, wrapHandler(onMouseOut), options);
    var enter = useEvent("mouseenter", ref, wrapHandler(onMouseEnter), options);
    var leave = useEvent("mouseleave", ref, wrapHandler(onMouseLeave), options);
    return mergeUseEventResults(down, move, up, over, out, enter, leave);
};
var useTouchEvents = function (_a, ref, options) {
    var onTouchStart = _a.onTouchStart, onTouchMove = _a.onTouchMove, onTouchEnd = _a.onTouchEnd, onTouchCancel = _a.onTouchCancel;
    var down = useEvent("touchstart", ref, wrapHandler(onTouchStart), options);
    var move = useEvent("touchmove", ref, wrapHandler(onTouchMove), options);
    var up = useEvent("touchend", ref, wrapHandler(onTouchEnd), options);
    var cancel = useEvent("touchcancel", ref, wrapHandler(onTouchCancel), options);
    return mergeUseEventResults(down, move, up, cancel);
};
var useNativePointerEvents = function (_a, ref, options) {
    var onPointerDown = _a.onPointerDown, onPointerMove = _a.onPointerMove, onPointerUp = _a.onPointerUp, onPointerCancel = _a.onPointerCancel, onPointerOver = _a.onPointerOver, onPointerOut = _a.onPointerOut, onPointerEnter = _a.onPointerEnter, onPointerLeave = _a.onPointerLeave;
    var down = useEvent("pointerdown", ref, wrapHandler(onPointerDown), options);
    var move = useEvent("pointermove", ref, wrapHandler(onPointerMove), options);
    var up = useEvent("pointerup", ref, wrapHandler(onPointerUp), options);
    var cancel = useEvent("pointercancel", ref, wrapHandler(onPointerCancel), options);
    var over = useEvent("pointerover", ref, wrapHandler(onPointerOver), options);
    var out = useEvent("pointerout", ref, wrapHandler(onPointerOut), options);
    var enter = useEvent("pointerenter", ref, wrapHandler(onPointerEnter), options);
    var leave = useEvent("pointerleave", ref, wrapHandler(onPointerLeave), options);
    return mergeUseEventResults(down, move, up, cancel, over, out, enter, leave);
};
var supportsPointerEvents = function () {
    return window.onpointerdown === null && window.onpointermove === null && window.onpointerup === null;
};
var supportsTouchEvents = function () {
    return window.ontouchstart === null && window.ontouchmove === null && window.ontouchend === null;
};
var supportsMouseEvents = function () {
    return window.onmousedown === null && window.onmousemove === null && window.onmouseup === null;
};
var usePointerEvents = function (eventHandlers, ref, options) {
    var onPointerDown = eventHandlers.onPointerDown, onPointerMove = eventHandlers.onPointerMove, onPointerUp = eventHandlers.onPointerUp, onPointerCancel = eventHandlers.onPointerCancel, onPointerOver = eventHandlers.onPointerOver, onPointerOut = eventHandlers.onPointerOut, onPointerEnter = eventHandlers.onPointerEnter, onPointerLeave = eventHandlers.onPointerLeave;
    var mouseEvents = {}, touchEvents = {}, pointerEvents = {};
    if (supportsPointerEvents()) {
        pointerEvents = eventHandlers;
    }
    else if (supportsTouchEvents()) {
        touchEvents = {
            onTouchStart: onPointerDown,
            onTouchMove: onPointerMove,
            onTouchEnd: onPointerUp,
            onTouchCancel: onPointerCancel,
        };
    }
    else if (supportsMouseEvents()) {
        mouseEvents = {
            onMouseDown: onPointerDown,
            onMouseMove: onPointerMove,
            onMouseUp: onPointerUp,
            onMouseOver: onPointerOver,
            onMouseOut: onPointerOut,
            onMouseEnter: onPointerEnter,
            onMouseLeave: onPointerLeave,
        };
    }
    var pointer = useNativePointerEvents(pointerEvents, ref, options);
    var touch = useTouchEvents(touchEvents, ref, options);
    var mouse = useMouseEvents(mouseEvents, ref, options);
    return mergeUseEventResults(pointer, touch, mouse);
};
var useConditionalPointerEvents = function (eventHandlers, ref, options) {
    var pointerEventsRef;
    var pointerEventsHandlers;
    var emptyRef = React.useRef(null);
    if (ref) {
        pointerEventsRef = ref;
        pointerEventsHandlers = eventHandlers;
    }
    else {
        pointerEventsRef = emptyRef;
        pointerEventsHandlers = {};
    }
    usePointerEvents(pointerEventsHandlers, pointerEventsRef, options);
    if (ref) {
        return undefined;
    }
    else {
        return eventHandlers;
    }
};

var Point;
(function (Point) {
    Point.subtract = function (a, b) {
        return { x: a.x - b.x, y: a.y - b.y };
    };
})(Point || (Point = {}));

function usePanGesture(_a, ref) {
    var onPan = _a.onPan, onPanStart = _a.onPanStart, onPanEnd = _a.onPanEnd;
    var session = null;
    var onPointerMove = React.useMemo(function () {
        return function (event, _a) {
            var point = _a.point, devicePoint = _a.devicePoint;
            if (!session) {
                console.error("Pointer move without started session");
                return;
            }
            var delta = Point.subtract(devicePoint, session.lastDevicePoint);
            var offset = Point.subtract(devicePoint, session.startDevicePoint);
            if (Math.abs(delta.x) > 0 || Math.abs(delta.y) > 0) {
                if (session.startEvent) {
                    if (onPan) {
                        onPan({ point: point, devicePoint: devicePoint, delta: delta, offset: offset }, event);
                    }
                }
                else {
                    if (onPanStart) {
                        onPanStart({ point: point, devicePoint: devicePoint, delta: delta, offset: offset }, event);
                    }
                    session.startEvent = event;
                }
            }
            session.lastDevicePoint = devicePoint;
        };
    }, [onPan, onPanStart]);
    var onPointerUp = React.useMemo(function () {
        return function (event, _a) {
            var point = _a.point, devicePoint = _a.devicePoint;
            if (!session) {
                console.error("Pointer end without started session");
                return;
            }
            var delta = Point.subtract(devicePoint, session.lastDevicePoint);
            var offset = Point.subtract(devicePoint, session.startDevicePoint);
            stopPointerMove();
            stopPointerUp();
            if (onPanEnd) {
                onPanEnd({ point: point, devicePoint: devicePoint, delta: delta, offset: offset }, event);
            }
            session = null;
        };
    }, [onPanEnd, onPointerMove]);
    var _b = usePointerEvents({ onPointerUp: onPointerUp }, window), startPointerUp = _b[0], stopPointerUp = _b[1];
    var _c = usePointerEvents({ onPointerMove: onPointerMove }, window, { capture: true }), startPointerMove = _c[0], stopPointerMove = _c[1];
    var onPointerDown = React.useMemo(function () {
        return function (event, _a) {
            var devicePoint = _a.devicePoint;
            session = {
                target: event.target,
                lastDevicePoint: devicePoint,
                startDevicePoint: devicePoint,
            };
            startPointerMove();
            startPointerUp();
        };
    }, [onPointerUp, onPointerMove]);
    React.useEffect(function () {
        return function () {
            stopPointerMove();
            stopPointerUp();
        };
    });
    return useConditionalPointerEvents({ onPointerDown: onPointerDown }, ref);
}

function useTapGesture(_a, ref) {
    var onTap = _a.onTap;
    var session = null;
    var onPointerUp = React.useMemo(function () {
        return function (event, _a) {
            var point = _a.point, devicePoint = _a.devicePoint;
            if (!session) {
                return;
            }
            if (!ref || event.target !== ref.current) {
                return;
            }
            if (onTap) {
                onTap({ point: point, devicePoint: devicePoint }, event);
            }
            session = null;
        };
    }, [onTap]);
    var onPointerDown = function (event) {
        startPointerUp();
        if (!ref || event.target !== ref.current)
            return;
        session = {
            target: event.target,
        };
    };
    var _b = usePointerEvents({ onPointerUp: onPointerUp }, window), startPointerUp = _b[0], stopPointerUp = _b[1];
    React.useEffect(function () {
        return function () {
            stopPointerUp();
        };
    }, [ref && ref.current, onPointerUp]);
    return useConditionalPointerEvents({ onPointerDown: onPointerDown }, ref);
}

var useGestures = function (props, ref) {
    usePanGesture(props, ref);
    useTapGesture(props, ref);
};

function createLock(name) {
    var lock = null;
    return function () {
        var openLock = function () {
            lock = null;
        };
        if (lock === null) {
            lock = name;
            return openLock;
        }
        return false;
    };
}

var draggableDefaults = {
    dragEnabled: false,
    dragLocksDirection: false,
    dragPropagation: false,
};
function defaults(props, defaultProps) {
    var result = defaultProps;
    Object.assign(result, props);
    return result;
}
function shouldDrag(direction, dragEnabled, currentDirection) {
    return ((dragEnabled === true || dragEnabled === direction) &&
        (currentDirection === null || currentDirection === direction));
}
function useDraggable(props, ref, values) {
    var _a = defaults(props, draggableDefaults), dragEnabled = _a.dragEnabled, dragPropagation = _a.dragPropagation, dragLocksDirection = _a.dragLocksDirection;
    var point = {};
    var currentDirection = null;
    if (shouldDrag("x", dragEnabled, currentDirection)) {
        point.x = values.get("x", 0);
    }
    if (shouldDrag("y", dragEnabled, currentDirection)) {
        point.y = values.get("y", 0);
    }
    var motionContext = React.useContext(MotionContext);
    var openGlobalLock = false;
    var onPanStart = React.useMemo(function () {
        return function () {
            if (!dragPropagation) {
                openGlobalLock = getGlobalLock(dragEnabled);
                if (!openGlobalLock) {
                    return;
                }
            }
            currentDirection = null;
            motionContext.dragging = true;
        };
    }, [dragEnabled, dragPropagation, motionContext]);
    var onPan = React.useMemo(function () {
        return function (_a) {
            var delta = _a.delta, offset = _a.offset;
            if (!dragPropagation && !openGlobalLock) {
                return;
            }
            if (dragLocksDirection) {
                if (currentDirection === null) {
                    currentDirection = getCurrentDirection(offset);
                    return;
                }
            }
            var x = point.x, y = point.y;
            if (shouldDrag("x", dragEnabled, currentDirection) && x) {
                x.set(x.get() + delta.x);
            }
            if (shouldDrag("y", dragEnabled, currentDirection) && y) {
                y.set(y.get() + delta.y);
            }
        };
    }, [openGlobalLock, dragEnabled, point]);
    var onPanEnd = React.useMemo(function () {
        return function () {
            if (!dragPropagation && openGlobalLock) {
                openGlobalLock();
            }
            motionContext.dragging = false;
        };
    }, [openGlobalLock, motionContext, dragEnabled]);
    var handlers = {};
    if (dragEnabled) {
        handlers = { onPanStart: onPanStart, onPan: onPan, onPanEnd: onPanEnd };
    }
    usePanGesture(handlers, ref);
}
function getCurrentDirection(offset) {
    var lockThreshold = 10;
    var direction = null;
    if (Math.abs(offset.y) > lockThreshold) {
        direction = "y";
    }
    else if (Math.abs(offset.x) > lockThreshold) {
        direction = "x";
    }
    return direction;
}
var globalHorizontalLock = createLock("dragHorizontal");
var globalVerticalLock = createLock("dragVertical");
function getGlobalLock(dragEnabled) {
    var lock = false;
    if (dragEnabled === "y") {
        lock = globalVerticalLock();
    }
    else if (dragEnabled === "x") {
        lock = globalHorizontalLock();
    }
    else {
        var openHorizontal_1 = globalHorizontalLock();
        var openVertical_1 = globalVerticalLock();
        if (openHorizontal_1 && openVertical_1) {
            lock = function () {
                openHorizontal_1();
                openVertical_1();
            };
        }
        else {
            if (openHorizontal_1)
                openHorizontal_1();
            if (openVertical_1)
                openVertical_1();
        }
    }
    return lock;
}

var createMotionComponent = function (Component) {
    var MotionComponent = function (p, externalRef) {
        var _a = p, animate = _a.animate, _b = _a.pose, pose = _b === void 0 ? "default" : _b, style = _a.style, onPoseComplete = _a.onPoseComplete, _c = _a.inherit, inherit = _c === void 0 ? false : _c, initialPose = _a.initialPose, dragEnabled = _a.dragEnabled, dragLocksDirection = _a.dragLocksDirection, dragPropagation = _a.dragPropagation, props = __rest(_a, ["animate", "pose", "style", "onPoseComplete", "inherit", "initialPose", "dragEnabled", "dragLocksDirection", "dragPropagation"]);
        var ref = useExternalRef(externalRef);
        var values = useMotionValues(ref);
        var controls = useAnimationControls(values, inherit, props);
        var context = useMotionContext(controls, inherit, initialPose || pose);
        useAnimationSubscription(animate, controls);
        usePoses(animate, inherit, controls, onPoseComplete, pose, initialPose);
        useGestures(props, ref);
        useDraggable({ dragEnabled: dragEnabled, dragLocksDirection: dragLocksDirection, dragPropagation: dragPropagation }, ref, values);
        return (React.createElement(MotionContext.Provider, { value: context }, React.createElement(Component, __assign({}, props, { ref: ref, style: useStyleAttr(values, style) }))));
    };
    return React.memo(React.forwardRef(MotionComponent));
};

var motion = elements.reduce(function (acc, element) {
    acc[element] = createMotionComponent(element);
    return acc;
}, { custom: createMotionComponent });

var useMotionValue = function (init) { return React.useMemo(function () { return motionValue(init); }, []); };

var useTransformedValue = function (value, from, to) {
    var transformedValue = React.useRef(null);
    return React.useMemo(function () {
        if (transformedValue.current)
            transformedValue.current.destroy();
        var transformer = popcorn.interpolate(from, to);
        transformedValue.current = value.addChild({ transformer: transformer });
        return transformedValue.current;
    }, [value].concat(from, to));
};

var useAnimation = function (poses) {
    var animation = React.useMemo(function () { return new AnimationManager(); }, []);
    poses && animation.setPoses(poses);
    React.useEffect(function () {
        animation.mount();
        return function () { return animation.unmount(); };
    }, []);
    return animation;
};

exports.motion = motion;
exports.useMotionValue = useMotionValue;
exports.useTransformedValue = useTransformedValue;
exports.useAnimation = useAnimation;
exports.useTapGesture = useTapGesture;
exports.usePanGesture = usePanGesture;
exports.useGestures = useGestures;
