export declare type Props = {
    [key: string]: any;
};
export declare type EasingFunction = (v: number) => number;
export declare type CubicBezier = [number, number, number, number];
export declare type Easing = CubicBezier | "linear" | "easeIn" | "easeOut" | "easeInOut" | "circIn" | "circOut" | "circInOut" | "backIn" | "backOut" | "backInOut" | "anticipate" | EasingFunction;
export declare type BaseTransition = {
    delay?: number;
    from?: number | string;
    to?: number | string;
    velocity?: number;
    beforeChildren?: boolean;
    afterChildren?: boolean;
    delayChildren?: number;
    staggerChildren?: number;
    staggerDirection?: 1 | -1;
    applyOnEnd?: PoseDefinition;
};
export declare type Tween = BaseTransition & {
    type?: "tween";
    duration?: number;
    ease?: Easing;
    elapsed?: number;
    loop?: number;
    flip?: number;
    yoyo?: number;
};
export declare type Spring = BaseTransition & {
    type: "spring";
    stiffness?: number;
    damping?: number;
    mass?: number;
    restSpeed?: number;
    restDelta?: number;
};
export declare type Decay = BaseTransition & {
    type: "decay";
    modifyTarget?: (v: number) => number;
    power?: number;
    timeConstant?: number;
    restDelta?: number;
};
export declare type Keyframes = BaseTransition & {
    type: "keyframes";
    values: number[] | string[];
    easings?: Easing[];
    easeAll?: Easing;
    elapsed?: number;
    duration?: number;
    loop?: number;
    flip?: number;
    yoyo?: number;
};
export declare type Physics = BaseTransition & {
    type: "physics";
    acceleration?: number;
    friction?: number;
    restSpeed?: number | false;
};
export declare type Just = BaseTransition & {
    type: "just";
};
export declare type Transition = Tween | Spring | Decay | Keyframes | Physics | Just;
export declare type PoseTransition = Tween | Spring | Decay | Keyframes | Physics | BaseTransition | false;
export declare type TransitionMap = {
    [key: string]: PoseTransition;
};
export declare type PoseDefinition = {
    x?: number | string;
    y?: number | string;
    z?: number | string;
    rotate?: number | string;
    rotateX?: number | string;
    rotateY?: number | string;
    rotateZ?: number | string;
    scale?: number | string;
    scaleX?: number | string;
    scaleY?: number | string;
    scaleZ?: number | string;
    skew?: number | string;
    skewX?: number | string;
    skewY?: number | string;
    originX?: number | string;
    originY?: number | string;
    originZ?: number | string;
    opacity?: number;
    perspective?: number | string;
    transform?: string;
    width?: number | string;
    height?: number | string;
    maxWidth?: number | string;
    maxHeight?: number | string;
    top?: number | string;
    left?: number | string;
    right?: number | string;
    bottom?: number | string;
    padding?: number | string;
    paddingTop?: number | string;
    paddingRight?: number | string;
    paddingBottom?: number | string;
    paddingLeft?: number | string;
    margin?: number | string;
    marginTop?: number | string;
    marginRight?: number | string;
    marginBottom?: number | string;
    marginLeft?: number | string;
    borderColor?: number | string;
    borderTopColor?: number | string;
    borderRightColor?: number | string;
    borderBottomColor?: number | string;
    borderLeftColor?: number | string;
    borderWidth?: number | string;
    borderTopWidth?: number | string;
    borderRightWidth?: number | string;
    borderBottomWidth?: number | string;
    borderLeftWidth?: number | string;
    borderRadius?: number | string;
    borderTopLeftRadius?: number | string;
    borderTopRightRadius?: number | string;
    borderBottomRightRadius?: number | string;
    borderBottomLeftRadius?: number | string;
    color?: string;
    backgroundColor?: string;
    outlineColor?: string;
    fill?: string;
    stroke?: string;
    background?: string;
    backgroundImage?: string;
    display?: string;
    position?: string;
    d?: string;
    pathLength?: number;
    pathSpacing?: number;
};
export declare type PoseAndTransition = [PoseDefinition, PoseTransition];
export declare type PoseResolver = (props: Props) => PoseDefinition | PoseAndTransition;
export declare type Pose = PoseDefinition | PoseAndTransition | PoseResolver;
export declare type Poses = {
    [key: string]: Pose;
};
